package blusunrize.immersiveengineering.common.util.inventory;

import net.minecraft.item.ItemStack;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.ItemHandlerHelper;

public class IEInventoryHandler implements IItemHandlerModifiable
{
	int slots;
	IIEInventory inv;
	int slotOffset;
	boolean[] canInsert;
	boolean[] canExtract;

	public IEInventoryHandler(int slots, IIEInventory inventory, int slotOffset, boolean[] canInsert, boolean[] canExtract)
	{
		this.slots = slots;
		this.inv = inventory;
		this.slotOffset = slotOffset;
		this.canInsert = canInsert;
		this.canExtract = canExtract;
	}
	public IEInventoryHandler(int slots, IIEInventory inventory)
	{
		this(slots,inventory,0, new boolean[slots], new boolean[slots]);
		for(int i=0; i<slots; i++)
			this.canExtract[i] = this.canInsert[i] = true;
	}
	public IEInventoryHandler(int slots, IIEInventory inventory, int slotOffset, boolean canInsert, boolean canExtract)
	{
		this(slots,inventory,slotOffset, new boolean[slots], new boolean[slots]);
		for(int i=0; i<slots; i++)
		{
			this.canInsert[i] = canInsert;
			this.canExtract[i] = canExtract;
		}
	}

	@Override
	public int getSlots()
	{
		return slots;
	}

	@Override
	public ItemStack getStackInSlot(int slot)
	{
		return this.inv.getInventory()[this.slotOffset + slot];
	}
	@Override
	public ItemStack insertItem(int slot, ItemStack stack, boolean simulate)
	{
		if(!canInsert[slot] || stack==null)
			return stack;
		stack = stack.func_77946_l();

		if(!inv.isStackValid(this.slotOffset + slot, stack))
			return stack;
		
		int offsetSlot = this.slotOffset+slot;
		ItemStack currentStack = inv.getInventory()[offsetSlot];

		if(currentStack==null)
		{
			int accepted = Math.min(stack.func_77976_d(), inv.getSlotLimit(offsetSlot));
			if(accepted<stack.field_77994_a)
			{
				if(!simulate)
				{
					inv.getInventory()[offsetSlot] = stack.func_77979_a(accepted);
					inv.doGraphicalUpdates(offsetSlot);
					return stack;
				}
				else
				{
					stack.field_77994_a -= accepted;
					return stack;
				}
			}
			else
			{
				if(!simulate)
				{
					inv.getInventory()[offsetSlot] = stack;
					inv.doGraphicalUpdates(offsetSlot);
				}
				return null;
			}
		}
		else
		{
			if(!ItemHandlerHelper.canItemStacksStack(stack, currentStack))
				return stack;

			int accepted = Math.min(stack.func_77976_d(), inv.getSlotLimit(offsetSlot)) - currentStack.field_77994_a;
			if(accepted<stack.field_77994_a)
			{
				if(!simulate)
				{
					ItemStack newStack = stack.func_77979_a(accepted);
					newStack.field_77994_a += currentStack.field_77994_a;
					inv.getInventory()[offsetSlot] = newStack;
					inv.doGraphicalUpdates(offsetSlot);
					return stack;
				}
				else
				{
					stack.field_77994_a -= accepted;
					return stack;
				}
			}
			else
			{
				if(!simulate)
				{
					ItemStack newStack = stack.func_77946_l();
					newStack.field_77994_a += currentStack.field_77994_a;
					inv.getInventory()[offsetSlot] = newStack;
					inv.doGraphicalUpdates(offsetSlot);
				}
				return null;
			}
		}
	}
	@Override
	public ItemStack extractItem(int slot, int amount, boolean simulate)
	{
		if(!canExtract[slot] || amount==0)
			return null;

		int offsetSlot = this.slotOffset+slot;
		ItemStack currentStack = inv.getInventory()[offsetSlot];

		if(currentStack==null)
			return null;

		int extracted = Math.min(currentStack.field_77994_a, amount);

		ItemStack copy = currentStack.func_77946_l();
		copy.field_77994_a = extracted;
		if(!simulate)
		{
			if(extracted<currentStack.field_77994_a)
				currentStack.field_77994_a -= extracted;
			else
				currentStack = null;
			inv.getInventory()[offsetSlot] = currentStack;
			inv.doGraphicalUpdates(offsetSlot);
		}
		return copy;
	}
	@Override
	public void setStackInSlot(int slot, ItemStack stack)
	{
		inv.getInventory()[this.slotOffset+slot] = stack;
		inv.doGraphicalUpdates(this.slotOffset+slot);
	}
}
